#pragma semicolon 1

#include <sourcemod>
#include <geoip>
#undef REQUIRE_EXTENSIONS
#include <clientprefs>
#undef REQUIRE_PLUGIN
#include <updater>

#define PLUGIN_NAME 	"GeoIP Language Selection"
#define PLUGIN_VERSION 	"1.2.0"

#define UPDATE_URL	"http://godtony.mooo.com/geolanguage/geolanguage.txt"

new Handle:g_hLangDB = INVALID_HANDLE;
new Handle:g_hCookie = INVALID_HANDLE;
new bool:g_bLoaded[MAXPLAYERS+1];
new bool:g_bUseCPrefs;

public Plugin:myinfo =
{
	name = PLUGIN_NAME,
	author = "GoD-Tony",
	description = "Automatically assign languages to players geographically",
	version = PLUGIN_VERSION,
	url = "http://www.sourcemod.net/"
};
 
public OnPluginStart()
{
	CreateConVar("sm_geolanguage_version", PLUGIN_VERSION, PLUGIN_NAME, FCVAR_PLUGIN|FCVAR_NOTIFY|FCVAR_DONTRECORD);
	RegConsoleCmd("sm_language", Command_Language);
	Init_DB();
	
	// Updater.
	if (LibraryExists("updater"))
	{
		Updater_AddPlugin(UPDATE_URL);
	}
}

public OnAllPluginsLoaded()
{
	// Ignoring the unlikely event where clientprefs is late-(re)loaded.
	if (LibraryExists("clientprefs"))
	{
		g_hCookie = RegClientCookie("GeoLanguage", "The client's preferred language.", CookieAccess_Protected);
		SetCookieMenuItem(CookieMenu_GeoLanguage, 0, "Language");
		g_bUseCPrefs = true;
	}
}

public OnLibraryAdded(const String:name[])
{
	if (StrEqual(name, "updater"))
	{
		Updater_AddPlugin(UPDATE_URL);
	}
}

public Updater_OnPluginUpdating()
{
	// Close the database handle before we attempt to overwrite.
	if (g_hLangDB != INVALID_HANDLE)
	{
		CloseHandle(g_hLangDB);
	}
}

public Updater_OnPluginUpdated()
{
	// We might be dealing with a new unknown database structure.
	ReloadPlugin();
}

public OnClientPutInServer(client)
{
	if (IsFakeClient(client))
		return;
	
	if (g_bUseCPrefs)
	{
		// If they aren't cached yet then we'll catch them on the cookie forward.
		if (AreClientCookiesCached(client) && !g_bLoaded[client])
		{
			LoadCookies(client);
		}
	}
	// CPrefs disabled. Set language without displaying help text.
	else if (GetClientLanguage(client) == 0)
	{
		SetClientLanguageByGeoIP(client);
	}
}

public OnClientCookiesCached(client)
{
	if (IsFakeClient(client))
		return;
	
	// If they aren't in-game yet then we'll catch them on the PutInServer forward.
	if (IsClientInGame(client) && !g_bLoaded[client])
	{
		LoadCookies(client);
	}
}

public OnClientDisconnect(client)
{
	g_bLoaded[client] = false;
}

public Action:Command_Language(client, args)
{
	/* The language command has been invoked. */
	if (client == 0)
	{
		ReplyToCommand(client, "[SM] This command is for players only.");
		return Plugin_Handled;
	}
	
	// Usage: sm_language
	if (args < 1)
	{
		LanguageMenu(client);
		return Plugin_Handled;
	}
	
	// Usage: sm_language <name>
	decl String:sLanguage[32], String:sCode[4];
	GetCmdArg(1, sLanguage, sizeof(sLanguage));
	new iLangID = GetLanguageByName(sLanguage);
	
	if (iLangID < 0)
	{
		ReplyToCommand(client, "[SM] Language not found: %s", sLanguage);
		return Plugin_Handled;
	}
	
	GetLanguageInfo(iLangID, sCode, sizeof(sCode), sLanguage, sizeof(sLanguage));
	SetClientLanguage(client, iLangID);
	
	if (g_bUseCPrefs)
	{
		SetClientCookie(client, g_hCookie, sCode);
	}
	
	ReplyToCommand(client, "[SM] Language changed to \"%s\".", sLanguage);
	return Plugin_Handled;
}

public Action:Timer_LanguageHelp(Handle:timer, any:userid)
{
	/* Tell the client that their language has been automatically set. */
	new client = GetClientOfUserId(userid);
	
	if (client == 0)
		return Plugin_Stop;
	
	decl String:sLanguage[32];
	GetLanguageInfo(GetClientLanguage(client), _, _, sLanguage, sizeof(sLanguage));
	
	PrintToChat(client, "[SM] Your language has been set to \"%s\". Type !language to change your language.", sLanguage);
	
	return Plugin_Stop;
}

public CookieMenu_GeoLanguage(client, CookieMenuAction:action, any:info, String:buffer[], maxlen) 
{
	/* Menu when accessed through !settings. */
	switch (action)
	{
		case CookieMenuAction_DisplayOption:
		{
			Format(buffer, maxlen, "Language");
		}
		case CookieMenuAction_SelectOption:
		{
			LanguageMenu(client);
		}
	}
}

LanguageMenu(client)
{
	/* Language selection menu. */
	new Handle:hMenu = CreateMenu(LanguageMenu_Handler);
	SetMenuTitle(hMenu, "Language:");
	
	new iCurrentLang = GetClientLanguage(client);
	new maxLangs = GetLanguageCount();
	decl String:sLanguage[32], String:sLangID[3];
	
	for (new i = 0; i < maxLangs; i++)
	{
		GetLanguageInfo(i, _, _, sLanguage, sizeof(sLanguage));
		IntToString(i, sLangID, sizeof(sLangID));
		
		if (i == iCurrentLang)
		{
			// Language already being used.
			Format(sLanguage, sizeof(sLanguage), "%s *", sLanguage);
			AddMenuItem(hMenu, sLangID, sLanguage, ITEMDRAW_DISABLED);
		}
		else
		{
			AddMenuItem(hMenu, sLangID, sLanguage, ITEMDRAW_DEFAULT);
		}
	}
	
	SetMenuExitButton(hMenu, true);
	DisplayMenu(hMenu, client, MENU_TIME_FOREVER);
}

public LanguageMenu_Handler(Handle:menu, MenuAction:action, param1, param2)
{
	/* Handle the language selection menu. */
	switch (action)
	{
		case MenuAction_End:
		{
			CloseHandle(menu);
		}
		case MenuAction_Select:
		{
			decl String:sLanguage[32], String:sCode[4], String:sLangID[3];
			new iLangID;
			
			GetMenuItem(menu, param2, sLangID, sizeof(sLangID));
			
			iLangID = StringToInt(sLangID);
			GetLanguageInfo(iLangID, sCode, sizeof(sCode), sLanguage, sizeof(sLanguage));
			SetClientLanguage(param1, iLangID);
			
			if (g_bUseCPrefs)
			{
				SetClientCookie(param1, g_hCookie, sCode);
			}
			
			PrintToChat(param1, "[SM] Language changed to \"%s\".", sLanguage);
		}
	}
}

Init_DB()
{
	/* Initialize our Country-Language database. */
	decl String:sPath[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, sPath, sizeof(sPath), "data/sqlite/languages.sq3");
	
	// Make sure the file exists, otherwise SourceMod will create a new blank database using our name.
	if (!FileExists(sPath))
	{
		SetFailState("File missing: %s", sPath);
	}

	// This is a read-only 30kb database. No need for config options (database.cfg).
	decl String:error[255];
	new Handle:kv = CreateKeyValues("");
	KvSetString(kv, "driver", "sqlite");
	KvSetString(kv, "database", "languages");
	g_hLangDB = SQL_ConnectCustom(kv, error, sizeof(error), false);
	CloseHandle(kv);
	
	if (g_hLangDB == INVALID_HANDLE)
	{
		SetFailState("%s", error);
	}
}

LoadCookies(client)
{
	/* Load the language selection data for this client. */
	decl String:sCookie[4];
	sCookie[0] = '\0';
	
	GetClientCookie(client, g_hCookie, sCookie, sizeof(sCookie));

	if (sCookie[0] != '\0')
	{
		SetClientLanguageByCode(client, sCookie);
	}
	// Only act on clients that haven't changed Steam's default language.
	else if (GetClientLanguage(client) == 0)
	{
		SetClientLanguageByGeoIP(client);
		CreateTimer(15.0, Timer_LanguageHelp, GetClientUserId(client), TIMER_FLAG_NO_MAPCHANGE);
	}
		
	g_bLoaded[client] = true;
}

SetClientLanguageByCode(client, const String:code[])
{
	/* Set a client's language based on the language code. */
	new iLangID = GetLanguageByCode(code);
	
	if (iLangID >= 0)
	{
		SetClientLanguage(client, iLangID);
	}
}

SetClientLanguageByGeoIP(client)
{
	/* Set a client's language relative to their country. */
	decl String:ip[17], String:ccode[4];
	
	if (!GetClientIP(client, ip, sizeof(ip)))
		return;
	
	if (!GeoipCode3(ip, ccode))
		return;
	
	new iLangID = GetLanguageByGeoIP(g_hLangDB, ccode);
	SetClientLanguage(client, iLangID);
}

GetLanguageByGeoIP(Handle:db, const String:ccode[])
{
	/* 
	* Retrieve the most popular language spoken in a given country.
	*
	* Defaults to English (0) if the language doesn't exist in languages.cfg
	* or if there is a problem retrieving the language.
	*/
	static Handle:hLangStmt = INVALID_HANDLE;
	
	if (hLangStmt == INVALID_HANDLE)
	{
		decl String:error[255];
		hLangStmt = SQL_PrepareQuery(
			db,
			"SELECT Language FROM languages WHERE CountryCode = ? AND IsOfficial = 1 ORDER BY Percentage DESC LIMIT 0,1;",
			error,
			sizeof(error));
			
		if (hLangStmt == INVALID_HANDLE)
			return 0;
	}
 
	SQL_BindParamString(hLangStmt, 0, ccode, false);
	
	if (!SQL_Execute(hLangStmt))
		return 0;

	if (SQL_FetchRow(hLangStmt))
	{
		decl String:sLanguage[32];
		SQL_FetchString(hLangStmt, 0, sLanguage, sizeof(sLanguage));
		
		new iLangID = GetLanguageByName(sLanguage);
		
		if (iLangID >= 0)
			return iLangID;
	}
	
	return 0;
}
